<?php
/*
Plugin Name: Div Layer Editor
Plugin URI: http://www.divlayer.com
Description: Visually Edit Your WordPress Themes from your Front-End Site.
Version: 1.2.5
Author: DIV Text, LLC
Author URI: http://www.divtext.com
License: GPLv2
*/

require_once 'php/libraries.php';

global $divle_isEnabled, $divle_selectedTheme;

$divle_isEnabled = (isset($_COOKIE['divle_enabled']) AND $_COOKIE['divle_enabled'] == 'true');

function divle_header_loader()
{
    global $divle_isEnabled;
    $hasAccess = divle_currUserHasAccess();
    if(!($hasAccess || DIVLE_RUN_AS_DEMO) && !$divle_isEnabled) return;
    $divleEnableParam = isset($_GET['divle-enable']) ? $_GET['divle-enable'] : '';
    if($divle_isEnabled && !($hasAccess || DIVLE_RUN_AS_DEMO) && $divleEnableParam != 'false') divle_redirectToDisableDivle();
    
    $pluginUrl =  plugins_url('', __FILE__);
    
    wp_register_script('divle_jui_core',      "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.core.min.js",      array('jquery'));
    wp_register_script('divle_jui_widget',    "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.widget.min.js",    array('jquery'));
    wp_register_script('divle_jui_position',  "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.position.min.js",  array('jquery'));
    wp_register_script('divle_jui_button',    "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.button.min.js",    array('jquery'));
    wp_register_script('divle_jui_mouse',     "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.mouse.min.js",     array('jquery'));
    wp_register_script('divle_jui_draggable', "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.draggable.min.js", array('jquery'));
    wp_register_script('divle_jui_resizable', "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.resizable.min.js", array('jquery'));
    wp_register_script('divle_jui_tabs',      "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.tabs.min.js",      array('jquery'));
    wp_register_script('divle_jui_dialog',    "{$pluginUrl}/js/jquery-ui-1.8.17/minified/jquery.ui.dialog.min.js",    array('jquery', 'divle_jui_core', 'divle_jui_widget', 'divle_jui_position', 'divle_jui_button', 'divle_jui_mouse', 'divle_jui_draggable', 'divle_jui_resizable', 'divle_jui_tabs'));
    
    //wp_register_script('divle_jui_dialog', "{$pluginUrl}/js/jquery-ui.min.js", array('jquery'));
    wp_register_script('divle_jscolor',  "{$pluginUrl}/js/jscolor/jscolor.js");
    wp_register_script('divle_jcookie',  "{$pluginUrl}/js/jquery.cookie.js",     array('jquery'));
	wp_register_script('divle_elemsel',  "{$pluginUrl}/js/elementselector-2.js", array('jquery'));
	wp_enqueue_script('divle_editor',    "{$pluginUrl}/js/divlayer_editor.js",   array('divle_jui_dialog', 'divle_jscolor', 'divle_jscolor', 'divle_jcookie', 'divle_elemsel'));
	
	echo '<link rel="stylesheet" href="'.plugins_url('/css/jqueryui/themes/base/jquery.ui.all.css', __FILE__).'" type="text/css" media="all" />' . "\n";
	echo '<link rel="stylesheet" href="'.plugins_url('/css/divlayer_editor.css', __FILE__).'" type="text/css" media="all" />' . "\n";
	
	$isEnabledStr = $divle_isEnabled  ? 'true' : 'false';
	$runAsDemoStr = DIVLE_RUN_AS_DEMO ? 'true' : 'false';
	$hasAccessStr = $hasAccess        ? 'true' : 'false';
	$js = '<script type="text/javascript">'. "\n";
	$js .= 'var divle_isEnabled = '.$isEnabledStr.';'."\n";
	$js .= 'var divle_pluginUrl = "'.$pluginUrl.'";'."\n";
	$js .= 'var divle_runAsDemo = '.$runAsDemoStr.';'."\n";
	$js .= 'var divle_userHasAccess = '.$hasAccessStr.';'."\n";
	$js .= '</script>'."\n";
	echo $js;
}

function divle_adminBarRender()
{
    global $divle_isEnabled, $wp_admin_bar;
    
    $backendUrl =  plugins_url('/php/backend.php', __FILE__);
    
    if($divle_isEnabled)
    {
        $title  = "Disable DLE";
        $href   = "{$backendUrl}?action=disable-dle";
        $aTitle = "Disable Div Layer Editor";
    }
    else
    {
        $title  = "Enable DLE";
        $href   = "{$backendUrl}?action=enable-dle";
        $aTitle = "Enable Div Layer Editor";
    }
    
    $wp_admin_bar->add_menu( array(
        'parent' => false,
        'id'     => 'divle-en-dis',
        'title'  => __($title),
        'href'   => $href,
        'meta'   => array( 'html' => '', 'class' => 'divle-en-dis', 'onclick' => '', 'target' => '_blank', 'title' => $aTitle)
    ));
}

function divle_themeTemplateFilter()
{
	return sanitize_file_name($_COOKIE['divle_preview_theme_template']);
}
function divle_themeStylesheetFilter()
{
	return sanitize_file_name($_COOKIE['divle_preview_theme_stylesheet']);
}

add_action('wp_enqueue_scripts',         'divle_header_loader');
add_action('wp_before_admin_bar_render', 'divle_adminBarRender');

if($divle_isEnabled AND !empty($_COOKIE['divle_preview_theme_template']) AND !empty($_COOKIE['divle_preview_theme_stylesheet']))
{
    add_filter('template',   'divle_themeTemplateFilter');
    add_filter('stylesheet', 'divle_themeStylesheetFilter');
}

// @todo arif: find out a way to call add_filter() for template and stylesheet if only divle_currUserHasAccess() returns true
