(function($) {
	
// global variables to stored changed css properties
var divle_cssChanges = {};
// elements that ignore divlayer when click on it
var divle_excludedSelectors = ['html', 
		                       'body', 
		                       '.ui-dialog', 
		                       '.ui-dialog *',
		                       '.divle-wpadminbar',
		                       '.divle-wpadminbar *',
		                       '.divle-redbord-owner', 
		                       '.divle-redbord-owner *', 
		                       '.divlayer-red-border'];

// store css properties categorized by its tab
divle_cssProperties = {
  text: [
    'font-family', 'font-size', 'font-style', 'font-weight',
    'text-align', 'text-decoration', 'text-indent', 'text-transform',
    'color',
    'list-style-image', 'list-style-position', 'list-style-type',
    'line-height', 'white-space', 'letter-spacing', 'word-spacing',
    'direction'
  ],
  border: [
    'border-collapse', 'border-spacing',
    'border-color', 'border-top-color', 'border-right-color', 'border-bottom-color', 'border-left-color',
    'border-style', 'border-top-style', 'border-right-style', 'border-bottom-style', 'border-left-style',
    'border-width', 'border-top-width', 'border-right-width', 'border-bottom-width', 'border-left-width',
    'outline-color', 'outline-style', 'outline-width'
  ],
  margin: [
    'margin-top', 'margin-right', 'margin-bottom', 'margin-left',
    'padding-top', 'padding-right', 'padding-bottom', 'padding-left',
    'overflow'
  ],
  background: [
    'background-attachment', 'background-color', 'background-image', 'background-position', 'background-repeat'
  ],
  positioning: [
    'display', 'float', 'width', 'height', 'min-width', 'min-height', 'max-width', 'max-height', 'position', 'top', 'right', 'bottom', 'left', 'vertical-align', 'z-index'
  ]
};

// store css options each css propertes have
divle_cssOptions = {
  'font-family': {
    options: ['Arial', 'Arial Black', 'Bookman Old Style', 'Comic Sans MS', 'Courier', 'Courier New', 'Georgia', 'Impact', 'MS Sans Serif', 'Tahoma', 'Times New Roman', 'Verdana'], // the dropdown
    optional: true // if it's not optional, user can't do nothing but select pre-defined option.
  },
  'font-size': {
    options: ['inherit'],
    optional: true
  },
  'font-style': {
    options: ['normal', 'italic', 'oblique', 'inherit'],
    optional: false
  },
  'font-weight': {
    options: ['normal', 'bold', 'bolder', 'lighter', '100', '200', '300', '400', '500', '600', '700', '800', '900', 'inherit'],
    optional: false
  },
  'text-align': {
    options: ['left', 'right', 'center', 'justify', 'inherit'],
    optional: false
  },
  'text-decoration': {
    options: ['none', 'underline', 'overline', 'line-through', 'blink', 'inherit'],
    optional: false
  },
  'text-indent': {
    options: ['inherit'],
    optional: true
  },
  'text-transform': {
    options: ['capitalize', 'uppercase', 'lowercase', 'none', 'inherit'],
    optional: false
  },
  'list-style-image': {
    options: ['none', 'inherit'],
    optional: true
  },
  'list-style-position': {
    options: ['inside', 'outside', 'inherit'],
    optional: false,
  },
  'list-style-type': {
    options: ['disc', 'circle', 'square', 'decimal', 'decimal-leading-zero', 'lower-roman', 'upper-roman', 'lower-greek', 'lower-latin', 'upper-latin', 'armenian', 'georgian', 'lower-alpha', 'upper-alpha', 'none', 'inherit'],
    optional: false,
  },
  'line-height': {
    options: ['normal', 'inherit'],
    optional: true
  },
  'white-space': {
    options: ['normal', 'pre', 'nowrap', 'pre-wrap', 'pre-line', 'inherit'],
    optional: false
  },
  'letter-spacing': {
    options: ['normal', 'inherit'],
    optional: true
  },
  'word-spacing': {
    options: ['normal', 'inherit'],
    optional: true
  },
  'direction': {
    options: ['ltr', 'rtl', 'inherit'],
    optional: false
  },

  'border-collapse': {
    options: ['collapse', 'separate', 'inherit'],
    optional: false
  },
  'border-spacing': {
    options: ['inherit'],
    optional: true
  },
  'border-style': {
    options: ['none', 'hidden', 'dotted', 'dashed', 'solid', 'double', 'groove', 'ridge', 'inset', 'outset'],
    optional: false
  },
  'border-top-style': {
    options: ['none', 'hidden', 'dotted', 'dashed', 'solid', 'double', 'groove', 'ridge', 'inset', 'outset'],
    optional: false
  },
  'border-right-style': {
    options: ['none', 'hidden', 'dotted', 'dashed', 'solid', 'double', 'groove', 'ridge', 'inset', 'outset'],
    optional: false
  },
  'border-bottom-style': {
    options: ['none', 'hidden', 'dotted', 'dashed', 'solid', 'double', 'groove', 'ridge', 'inset', 'outset'],
    optional: false
  },
  'border-left-style': {
    options: ['none', 'hidden', 'dotted', 'dashed', 'solid', 'double', 'groove', 'ridge', 'inset', 'outset'],
    optional: false
  },
  'border-width': {
      options: ['inherit'],
      optional: true
  },
  'border-top-width': {
      options: ['inherit'],
      optional: true
  },
  'border-right-width': {
      options: ['inherit'],
      optional: true
  },
  'border-bottom-width': {
      options: ['inherit'],
      optional: true
  },
  'border-left-width': {
      options: ['inherit'],
      optional: true
  },
  'outline-style': {
      options: ['inherit'],
      optional: true
  },
  'outline-width': {
      options: ['inherit'],
      optional: true
  },
  'margin-top': {
      options: ['inherit'],
      optional: true
  },
  'margin-right': {
      options: ['inherit'],
      optional: true
  },
  'margin-bottom': {
      options: ['inherit'],
      optional: true
  },
  'margin-left': {
      options: ['inherit'],
      optional: true
  },
  'padding-top': {
      options: ['inherit'],
      optional: true
  },
  'padding-right': {
      options: ['inherit'],
      optional: true
  },
  'padding-bottom': {
      options: ['inherit'],
      optional: true
  },
  'padding-left': {
      options: ['inherit'],
      optional: true
  },
  'overflow': {
      options: ['visible', 'hidden', 'scroll', 'auto', 'inherit'],
      optional: false
  },

  'background-attachment': {
      options: ['scroll', 'fixed', 'inherit'],
      optional: false
  },
  'background-image': {
      options: ['none', 'inherit'],
      optional: true
  },
  'background-position': {
      options: ['left', 'center', 'right', 'top', 'center', 'bottom', 'inherit'],
      optinal: true
  },
  'background-repeat': {
      options: ['repeat', 'repeat-x', 'repeat-y', 'no-repeat', 'inherit'],
      optinal: false
  },

  'display': {
      options: ['inline', 'block', 'list-item', 'inline-block', 'table', 'inline-table', 'table-row-group', 'table-header-group', 'table-footer-group', 'table-row', 'table-column-group', 'table-column', 'table-cell', 'table-caption', 'none', 'inherit'],
      optional: false
  },
  'float': {
      options: ['left', 'right', 'none', 'inherit'],
      optional: false
  },
  'width': {
      options: ['auto', 'inherit'],
      optional: true
  },
  'height': {
      options: ['auto', 'inherit'],
      optional: true
  },
  'min-width': {
      options: ['inherit'],
      optional: true
  },
  'min-height': {
      options: ['inherit'],
      optional: true
  },
  'max-width': {
      options: ['none', 'inherit'],
      optional: true
  },
  'max-height': {
      options: ['none', 'inherit'],
      optional: true
  },
  'position': {
      options: ['static', 'relative', 'absolute', 'fixed', 'inherit'],
      optional: false
  },
  'top': {
      options: ['auto', 'inherit'],
      optional: true
  },
  'right': {
      options: ['auto', 'inherit'],
      optional: true
  },
  'bottom': {
      options: ['auto', 'inherit'],
      optional: true
  },
  'left': {
      options: ['auto', 'inherit'],
      optional: true
  },
  'vertical-align': {
      options: ['baseline', 'sub', 'super', 'top', 'text-top', 'middle', 'bottom', 'text-bottom', 'inherit'],
      optional: true
  },
  'z-index': {
      options: ['auto', 'inherit'],
      optional: true
  }
};

// list of css properties that require use of jscolor library
divle_cssTypeProperties = {
  color: ['color', 'border-color', 'border-top-color', 'border-right-color', 'border-bottom-color', 'border-left-color', 'outline-color', 'background-color']
};

/**
 * Log changes of element's properties.
 * @param string selector element's css selector
 * @param string property css property to be added or updated
 * @param string value new value of assigned css property
 */
divle_addCssChanges = function (selector, property, value) {
    if($('#divlayer-action-check').hasClass('active'))
    {
    	var multiselSelectors = $('#divlayer-dialog').data('multisel-selectors');
    	$.each(multiselSelectors, function(i, mSelector){
    	    // no specified selector defined, go define it
    	    if(divle_cssChanges[mSelector] == undefined) divle_cssChanges[mSelector] = {};
    	    divle_cssChanges[mSelector][property] = value;
    	});
    }
    else
    {
	    // no specified selector defined, go define it
	    if(divle_cssChanges[selector] == undefined) divle_cssChanges[selector] = {};
	    divle_cssChanges[selector][property] = value;
    }
}

/**
 * Parse list of css properties into tab.
 */
divle_parseCssPropertiesTabs = function() {
  // iterate css property categories
  for(var key in divle_cssProperties) {
    // parse every one of them
    divle_parseCssPropertiesForm(key);
  }
}

/**
 * Parse collection of css property at one tab into form
 * @param string tab the tab identifier
 */
divle_parseCssPropertiesForm = function(tab){
  // no tab specified, use default value
  if(tab == undefined) tab = 'text';

  // get properties based on tab
  var properties = divle_cssProperties[tab];

  // no properties found, raise an error
  if(properties == undefined) jQuery.error('No css property tab found.');

  // prepare form placeholder
  var jQuerytable = jQuery('<table/>');

  // make every properties an input field
  jQuery.each(properties, function(i, property){
      // prepare input placeholder
      var jQuerydeclaration = jQuery('<tr/>');
      var jQueryproperty = jQuery('<td class="divlayer-css-property"/>');
      jQueryproperty
        .text(property)
        .appendTo(jQuerydeclaration)
      var jQueryvalue = jQuery('<td class="divlayer-css-value"/>');
      jQueryvalue
        .appendTo(jQuerydeclaration);

      // craete the input
      var jQueryinput = divle_getCssValueInput(property);
      jQueryinput
        .appendTo(jQueryvalue);
      jQuerydeclaration.appendTo(jQuerytable);
  });

  // put the result into tab's container
  jQuery('#divlayer-' + tab).html(jQuerytable);
}

/**
 * Create dropdown for css properties which have pre-defined options.
 * @param object jQueryelement a jQuery element object of input field
 * @param object options options gotten from divle_cssProperties
 */
divle_createDropdown = function(jQueryelement, options) {
  // prepare dropdown placeholder
  var jQuerydropdown = jQuery('<ul class="divlayer-dropdown" />');

  // iterate over options, generate selection for each option value
  jQuery.each(options, function(i, option){
      // prepare option placeholder
      var jQueryoption = jQuery('<li class="divlayer-dropdown-option" />');
      jQueryoption
        .text(option)
        .hover(function(){
          // set "mouseover" event to the option

          // remove other options highlight state
          jQuery(this).siblings('.hovered').removeClass('hovered');

          // highlight current option
          jQuery(this).addClass('hovered');

        }, function(){
          // set "mouseout" event to the option

          // remove highlight from current option
          jQuery(this).removeClass('hovered');
        })
        .click(function(){
          // set "click" event to the option

          // get option's text
          var text = jQuery(this).text();

          // prepare valid css property's value
          text = divle_sanitizeCssTextValue(text)

          // set the element to selected option
          jQueryelement.val(text);

          // trigger "change" event so changes will be noticed
          jQueryelement.trigger('change');
        })
        .appendTo(jQuerydropdown);
  });

  // insert the dropdown after the input element
  jQuerydropdown.insertAfter(jQueryelement);
}

/**
 * Return string with single quotes at the beginning and the end of the string if the string contains space.
 * @param string text string to be sanitized
 */
divle_sanitizeCssTextValue = function(text){
  if(text.indexOf(' ') != -1)
    // there are some spaces, cover it with single quotes
    text = "'" + text + "'";
  return text;
}

/**
 * Create input based on assigned css property.
 * @param string property css property
 * @return object the input element
 */
divle_getCssValueInput = function(property){
  // get currently selected element
  var jQueryelement = jQuery('#divlayer-dialog').data('element');

  // create input element and set the property name to its data variable
  var jQueryinput = jQuery('<input type="text" class="divlayer-css-value-input" />');
  jQueryinput.data('property', property);

  if(divle_cssOptions[property] != undefined) {
    // the css property has some options
    var options = divle_cssOptions[property].options;

    // whether the property is optional
    var isOptional = divle_cssOptions[property].optional;

    // set input's data variable
    jQueryinput
      .data('options', options)
      .focus(function(){
        // set "focus" event to the input
        // it it's on focus, it will show the css property's list of options

        var jQuerythis = jQuery(this);
        // save previous value in case new value is invalid, we can revert the value back
        jQuerythis.data('previous-value', jQuerythis.val());

        // get the option dropdown element
        var jQuerydropdown = jQuerythis.next('.divlayer-dropdown');
        if(jQuerydropdown.length == 0) {
          // no dropdown found, create one with specified element and optons
          divle_createDropdown(jQueryinput, options);
          // get the option dropdown, once again
          jQuerydropdown = jQuerythis.next('.divlayer-dropdown');
        }

        // show the dropdown
        jQuerydropdown.show();
        jQuerydropdown.children('.hovered').removeClass('hovered');

        // selected option sometimes has single quotes in it, remove it
        var val = jQuerythis.val().replace(/\'/g, '');

        // if selected options if valid (exist in options dictionary) then highlight it
        var indexVal = jQuery.inArray(val, options);
        if(indexVal != -1) jQuerydropdown.children(':eq('+indexVal+')').addClass('hovered');

      })
      .blur(function(){
        // set "blur" event to the input
        // remove the dropdown
        var jQuerythis = jQuery(this);

        // why 100 miliseconds delay? if no delay specified, we won't have chance to select the option
        // as the option list dissappear immediately without delay
        setTimeout(function(){
          jQuerythis.next('.divlayer-dropdown').hide();
        }, 100)

      })
      .keydown(function(e){
        // set "keydown" event to the input
        var jQuerythis = jQuery(this);

        // select for dropdown option
        var jQuerydropdown = jQuery(this).next('.divlayer-dropdown');
        // it's not found, no further action
        if(jQuerydropdown.length == 0) return false;

        // check currently selected option item
        var jQueryselectedLi = jQuerydropdown.children('.hovered');
        var selectedIndex = jQueryselectedLi == undefined ? -1 : jQueryselectedLi.index();

        if(e.keyCode == 40) {
          // down arrow is pressed, move the highligt to the lower position
          selectedIndex += 1;
          jQueryselectedLi.removeClass('hovered');
          var jQuerynextLi = jQuerydropdown.children(':eq('+selectedIndex+')')

          // current option is on the bottom, the highlight now go to the top
          if(jQuerynextLi.length == 0) jQuerynextLi = jQuerydropdown.children(':first-child');
          jQuerynextLi.addClass('hovered');

          // the input needs to change its value and trigger "change" event so the change will be applied
          jQuerythis.val(divle_sanitizeCssTextValue(jQuerynextLi.text()));
          jQuerythis.trigger('change');

        } else if(e.keyCode == 38) {
          // up arrow is pressed, move the highlight to the upper position
          selectedIndex -= 1;
          jQueryselectedLi.removeClass('hovered');
          var jQueryprevLi = jQuerydropdown.children(':eq('+selectedIndex+')');

          // current option is on the top, the highlight now go to the bottom
          if(jQueryprevLi.length == 0) jQueryprevLi = jQuerydropdown.children(':last-child');
          jQueryprevLi.addClass('hovered');

          // the input needs to change its value and trigger "change" event so the change will be applied
          jQuerythis.val(divle_sanitizeCssTextValue(jQueryprevLi.text()));
          jQuerythis.trigger('change');

        } else if(e.keyCode == 27) {
          // esc key is pressed, change won't be made

          // use previous value if it's exist
          var previousValue = jQuerythis.data('previous-value');
          if(previousValue != undefined) jQuerythis.val(previousValue);

          // the "blur" event will close the option list
          jQuerythis.trigger('blur');
          return false;

        }

      })
      .keyup(function(e){
        // when key is released
        if(e.keyCode == 13) {
          // when enter key is released

          // get highlighted option
          var jQueryselectedItem = jQuery(this).next('.divlayer-dropdown').children('.hovered');

          // if it has no highlighted item, no further action
          if(jQueryselectedItem == undefined) return false;

          // trigger "click" event so the input will select the selected item
          jQueryselectedItem.trigger('click');

          // force to close the option list
          jQuery(this).trigger('blur');
        }

      })
      .change(function(){
        // if the input has its value changed, check whether it's valid or not
        var jQuerythis = jQuery(this);
        var value = jQuerythis.val();

        // the new value must exist in default options list
        if(isOptional == false && jQuery.inArray(value, options) == -1) {
          // if not, revert it back to its previous value
          value = jQuerythis.data('previous-value');
          jQuerythis.val(value);
        }

        // log the change
        var elementSelector = jQuery('#divlayer-dialog').data('element-selector');
        var property = jQuerythis.data('property');
        divle_addCssChanges(elementSelector, property, value);
      });
  }

  // get css propery's value of currently selected element
  var value = jQueryelement.css(property);

  if(value != undefined) {

    // check whether property is a color value
    if(jQuery.inArray(property, divle_cssTypeProperties['color']) != -1) {
      if(value.indexOf('rgba') == 0) {
        // the value format is like: rgba(255, 255, 255, 0), convert it correctly
        value = divle_RGBtoHex(value, true);
      } else if(value.indexOf('rgb') === 0) {
        // the value format is like: rgb(255, 255, 255), convert it correctly
        value = divle_RGBtoHex(value);
      }

      // instantiate new colorpicker element
      var picker = new jscolor.color(jQueryinput.get(0), {pickerZIndex: 9999999, hash: true});

      // set its default value
      picker.fromString(value);

      jQueryinput.change(function(){
        // if input value is changed, log the change
        var elementSelector = jQuery('#divlayer-dialog').data('element-selector');
        var property = jQuery(this).data('property');
        var value = jQuery(this).val();
        divle_addCssChanges(elementSelector, property, value);
      });
    } else {
      // change input's value normally
      jQueryinput.val(value);
    }
  }

  return jQueryinput;
}

/** 
 * Convert rgb to hexadecimal value.
 * @param string rgb the rgb value, like: rgb(255, 255, 255)
 * @param boolean withAlpha whether the rgb param has alpha-transparency specified
 * @return string like: #AADDEE
 */
divle_RGBtoHex = function(rgb, withAlpha) {
  // set withAlpha default to true
  if(withAlpha == undefined) withAlpha == true;

  // sanitize rgb core of css
  var rgb = rgb.replace(/[^\d\,\.]+/g, '')
  var hex = '';
  var rgbPart = rgb.split(',');

  // does rgb value set prevously? if not, return white (#ffffff)
  if(rgbPart.length == 4 && parseInt(rgbPart[3]) == 0) return '#FFFFFF';

  // turn array of rgb part into hexadecimal string value
  jQuery.each(rgbPart, function(i, hexval){
      // convert the integer into hexadecimal
      hexval = parseInt(hexval).toString(16);

      // if the hexadigimal only has one digit, prepend it with 0
      if(hexval.length == 1) hexval = "0" + hexval;
      hex += hexval;

      // alpha-transparency is ignored
      if(withAlpha && i == 2) return false;
  })

  return '#'+hex;
}

divle_enable = function() {
  // bind every element except the excluded
  jQuery("*").not(divle_excludedSelectors.join(',')).bind('click', divle_open);
  jQuery('.divle-en-dis a').text('Disable DLE').attr('href', '#').attr('title', 'Disable Div Layer Editor');
  divle_isEnabled = true;
  jQuery.cookie('divle_enabled', 'true');
}

divle_disable = function() {
  // unbind every element except the excluded
	jQuery("*").not(divle_excludedSelectors.join(',')).unbind('click', divle_open);
	jQuery('.divle-en-dis a').text('Disabling . .').attr('href', '#').attr('title', 'Disabling Div Layer Editor . .');
	divle_isEnabled = false;
	jQuery.cookie('divle_enabled', 'false');
	jQuery.cookie('divle_preview_theme_template',   null);
	jQuery.cookie('divle_preview_theme_stylesheet', null);
	
	var url    = document.URL;
	var newUrl = url.replace('divle-enable=false', '').replace('divle-enable=true', '');
	if(newUrl == url) window.location.reload();
	else window.location = newUrl;
}

// open div layer editor
divle_open = function(event) {
    event.stopPropagation(); // stop event's propagation
    event.preventDefault();  // prevents event's default behavior
    
    var jQuerythis = jQuery(this);
    
    if(jQuerythis.is(divle_excludedSelectors.join(','))) return false;
    
    // close divlayer's dialog
    jQuery('#divlayer-dialog').dialog('close');
    
    // remove the red block used for element highlighting
    divle_removeRedBorder();
    
    // highlight selected element
    divle_createRedBorder(jQuerythis);

    // set divlayer dialog's data variable with jQuery element and css selector of the element
    var jQuerydialog = jQuery('#divlayer-dialog');
    jQuerydialog
      .data('element', jQuerythis)
      .data('element-selector', divle_getElementSelector(jQuerythis))

    if(jQuerydialog.is(':visible')) {
      // dialog is already opened, reload it
      divle_reloadDialog();
    } else {
      // dialog is not opened, open it
      jQuerydialog.dialog('open');
    }
}

/**
 * Reload content of divlayer dialog.
 * @param object jQueryelem a jQuery element object
 */
divle_reloadDialog = function(jQueryelem) {
  var jQuerydialog = jQuery('#divlayer-dialog');

  // clean up behavior of previously selected element
  var jQueryoldElem = jQuerydialog.data('element');

  if(jQueryoldElem.is('table')) {
    // update the state of the table element
    jQueryoldElem.tableResize('update');
  } else if(jQueryoldElem.is('th') || jQueryoldElem.is('td')) {
    // temporarily disable cell resize plugin
    jQueryoldElem.tableCellResize('disable');
  } else if(jQueryoldElem.is('div')) {
    // disable resizing
    jQueryoldElem.resizable('destroy');
  }

  if(jQueryelem != undefined) {
    // if jQueryelem is assigned, store it to dialog's cache variable
    jQuery('#divlayer-dialog')
      .data('element', jQueryelem)
      .data('element-selector', divle_getElementSelector(jQueryelem))
  }

  // load main divlayer editor functionality
  var editorsrc =  divle_pluginUrl+'/php/editor.php';
  
  jQuerydialog.load(editorsrc, {nocache: 1}, function() { // {nocache: 1} is to force POST call

	divle_runEditorScripts();
	
    var elementSelectors = jQuerydialog.data('element-selectors');

    if(elementSelectors == undefined || elementSelectors.length == 0) {
      // no element selector specified

      // disable check button
      jQuery('#divlayer-action-check').removeClass('active');
      
      divle_closeMultiSelect();

      // enable up arrow button
      jQuery('#divlayer-arrow-up').removeClass('disabled');

      // enable down arrow button
      jQuery('#divlayer-arrow-down').removeClass('disabled');

    } else {

      // set check button state to active
      jQuery('#divlayer-action-check').addClass('active');
      
      divle_openMultiSelect();

      // disable up arrow button
      jQuery('#divlayer-arrow-up').addClass('disabled');

      // disable down arrow button
      jQuery('#divlayer-arrow-down').addClass('disabled');
    }
    
    if(divle_selectedParStylesheet == divle_selectedStylesheet || divle_selectedTemplate == divle_selectedStylesheet || divle_currStylesheet == divle_selectedStylesheet || divle_runAsDemo) // parent (in child group) || parent || current || demo
    {
    	jQuery('#divlayer-button-del').addClass('disabled');
    }
    if(divle_selectedParStylesheet == divle_selectedStylesheet || divle_selectedTemplate == divle_selectedStylesheet || divle_runAsDemo) // parent (in child group) || parent || demo
    {
    	jQuery('#divlayer-button-save').addClass('disabled');
    }
    if(divle_currStylesheet == divle_selectedStylesheet || divle_runAsDemo) // current || demo
    {
    	jQuery('#divlayer-action-version-check').addClass('disabled');
    }
    if(divle_runAsDemo) // demo
    {
    	jQuery('#divlayer-button-new').addClass('disabled');
    }

    // put element selector to text box
    var classSelector = jQuerydialog.data('class-selector');
    if(typeof classSelector == 'string' && classSelector != '') {
      // class selector is specified

      var elementSelector = classSelector;

      // set check button state to active and enable it (if it was disabled)
      jQuery('#divlayer-action-check')
        .addClass('active')
        .removeClass('disabled')

      // disable up arrow
      jQuery('#divlayer-arrow-up').addClass('disabled');

      // disable down arrow
      jQuery('#divlayer-arrow-down').addClass('disabled');

    } else {

      var elementSelector = jQuerydialog.data('element-selector');
    }

    // set search input to css selector
    jQuery('#divlayer-search-element').val(elementSelector);

    var jQueryelement = jQuerydialog.data('element');

    // disable up navigation if no parent found or this is a body element
    if(jQueryelement.is('html') || jQueryelement.is('body')) {
      jQuery('#divlayer-arrow-up').addClass('disabled');
    }

    // disable down navigation if no children found
    if(jQueryelement.children().length == 0) {
      jQuery('#divlayer-arrow-down').addClass('disabled');
    }

    // disable left and right navigation if no siblings found
    if(jQueryelement.siblings().length == 0) {
      jQuery('#divlayer-arrow-left, #divlayer-arrow-right').addClass('disabled');
    }

    // disable checkbox if no class found
    var excludedClassNames = ['', 'ui-resizable'];
    var elementClasses = jQuery.trim(jQueryelement.attr('class'));
    if(elementClasses == undefined || jQuery.inArray(elementClasses, excludedClassNames) != -1) {
      jQuery('#divlayer-action-check').addClass('disabled');
    }

    if(jQueryelement.is('div')) {
      // if element is a div, make it resizable

      if(jQueryelement.hasClass('ui-resizable')) {
        // if element is already resizable (but disabled), just enable it
        jQueryelement.resizable('enable');
      } else {
        // also resize its siblings
        jQueryelement.resizable({alsoResize: jQueryelement.siblings()});
      }

    } else if(jQueryelement.is('table')) {
      // if element is a table, make it resizable with tableResize plugin

      if(jQueryelement.data('tbl-id') == undefined) {
          // the plugin is not instantiated
          jQueryelement.tableResize();
      } else {
          // the plugin is already instantiated but disabled
          // enable it
          jQueryelement.tableResize('enable');
      }

    } else if(jQueryelement.is('th') || jQueryelement.is('td')) {
      // if element is a table cell (th/td), make it resizable with tableCellResize plugin

      if(jQueryelement.hasClass('ui-resizable')) {
        // the plugin is already instantiated but disabled
        // enable it
        jQueryelement.tableCellResize('enable');
      } else {
        // the plugin is not instantiated
        jQueryelement.tableCellResize();
      }
    }

    // create theme group name list
    var grpNameSelO = jQuery('#divlayer-input-theme-name');
    for(var i = 0; i < divle_themeGroups.length; i++)
    {
    	var themeGroup = divle_themeGroups[i];
    	grpNameSelO.append('<option value="'+themeGroup['template']+'">'+themeGroup['name']+'</option>');
    	
    	for(var j = 0; j < themeGroup['child_groups'].length; j++)
    	{
    		var childGroup = themeGroup['child_groups'][j];
    		grpNameSelO.append('<option value="'+themeGroup['template']+':'+childGroup['par_stylesheet']+'">'+'--'+childGroup['name']+'</option>');
    	}
    }
    var grpNameSelOVal = (divle_selectedParStylesheet.length > 0)  ? divle_selectedTemplate + ':' + divle_selectedParStylesheet : divle_selectedTemplate;
    grpNameSelO.val(grpNameSelOVal);
    grpNameSelO.change(function(){
    	divle_populateThemeGroupVersions();
    });
    
    // create theme group version list
    var grpVerSelO = jQuery('#divlayer-input-theme-version');
    divle_populateThemeGroupVersions();
    grpVerSelO.val(divle_selectedStylesheet);
    grpVerSelO.change(function(){
    	var stylesheet = jQuery(this).val();
    	if(stylesheet != "")
    	{
        	jQuery.cookie('divle_preview_theme_template',   divle_getTemplateInThemeDropdn());
        	jQuery.cookie('divle_preview_theme_stylesheet', stylesheet);
        	jQuery.cookie('divle_open_for_element',         jQuery('#divlayer-dialog').data('element-selector'));
        	
        	divle_switchBtnViewToLoading('#divlayer-arrow-round');
        	window.location.reload();
    	}
    })
    
    // alternative method if above functionality doesn't work
    jQuery('#divlayer-arrow-round').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      // trigger "change" to version selector so it will reload theme's version with specified theme version
      jQuery('#divlayer-input-theme-version').trigger('change');

      return false;
    });

    // set to current version event handler
    jQuery('#divlayer-action-version-check').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      if(jQuery(this).hasClass('disabled')) {
        // button is currently disabled, no further action
        return false;
      }

      divle_switchBtnViewToLoading('#divlayer-action-version-check');

      // save existing theme version
      var backendsetsrc = divle_pluginUrl+'/php/backend.php?action=set';
      
      jQuery.post(backendsetsrc, {nocache: 1}, function(response){

        if(response.success == true)
        {
          divle_reloadDialog();
        }
        else
        {
        	alert(response.meta.error_msg);
        }

      }, 'json');

      return false;

    });

    // event handler for save button
    jQuery('#divlayer-button-save').click(function(e){
      e.preventDefault();
      
      if(jQuery(this).hasClass('disabled')) {
          // button is currently disabled, no further action
          return false;
      }

      // save changes of currently selected theme version
      divle_saveChanges('save');

      return false;
    });
    // event handler for new button
    jQuery('#divlayer-button-new').click(function(e){
      e.preventDefault();
      
      if($(this).hasClass('disabled')) {
          // button is currently disabled, no further action
          return false;
      }

      // create new theme version with all changes implemented
      divle_saveChanges('new');

      return false;
    });

    // set to delete version event handler
    jQuery('#divlayer-button-del').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      if(jQuery(this).hasClass('disabled')) {
        // button is currently disabled, no further action
        return false;
      }

      divle_switchBtnViewToLoading('#divlayer-button-del');

      // delete selected theme version
      jQuery.post(divle_pluginUrl+'/php/backend.php?action=delete', {nocache: 1}, function(response){

        if(response.success == true) {
          jQuery.cookie('divle_preview_theme_template',   null);
          jQuery.cookie('divle_preview_theme_stylesheet', null);
          jQuery.cookie('divle_open_for_element', jQuery('#divlayer-dialog').data('element-selector'));
          window.location.reload();
        }
        else
        {
        	alert(response.meta.error_msg);
        }

      }, 'json');

      return false;
    });

  });
}

/**
 * Save css changes permanently
 * @param string trigger action that trigger this function
 */
divle_saveChanges = function(trigger){
    // get theme name and version
    var theme = divle_getTemplateInThemeDropdn();
    var themeVersion = parseInt(jQuery('#divlayer-input-theme-version').val());

    // save existing theme's version or create new one
    var obj = (trigger == 'save') ? '#divlayer-button-save' : '#divlayer-button-new';
    divle_switchBtnViewToLoading(obj);
    
    var backendsavesrc = divle_pluginUrl+'/php/backend.php?action=save';
    jQuery.post(backendsavesrc, {theme: theme, version: themeVersion, css: divle_cssChanges, trigger: trigger}, function(response){

      if(response.success == true) {
        if(typeof response.meta.new_version != 'undefined') {
          jQuery.cookie('divle_preview_theme_stylesheet', response.meta.new_stylesheet);
        }
        divle_cssChanges = {};
        divle_reloadDialog();
      }
      else
      {
    	  alert(response.meta.error_msg);
      }

    }, 'json');
}

/**
 * Highligt an element by creating a red border and move scroll to it.
 * @param object jQueryelem a jQuery element object
 */

divle_highlightElement = function(jQueryelem) {
  // highlight the element with red box
  divle_createRedBorder(jQueryelem);

  // reload the dialog
  divle_reloadDialog(jQueryelem);

  // calculate the element's location, then move the scroll to it
  var height = jQueryelem.outerHeight();
  var bodyHeight = jQuery('body').height()
  var offsetTop = jQueryelem.offset().top;
  if(height > bodyHeight) {
    var scrollTop = offsetTop - 30;
  } else {
    var scrollTop = offsetTop - ((bodyHeight - height) / 2);
  }

  if(scrollTop <= 0) scrollTop = 0 + 20;
  jQuerydialog = jQuery('#divlayer-dialog');
  jQuerydialog.parent().offset({top: scrollTop + 20});
  jQuery('body').animate({scrollTop: scrollTop + 'px'});

}

/**
 * Close dialog and clear or disable element's behaviour
 */
divle_closeDialog = function(){
  var jQueryelem = jQuery('#divlayer-dialog').data('element');

  // remove the highlighting
  divle_removeRedBorder();

  if(jQueryelem != undefined && jQueryelem.length != 0) {
    // if selected element still exist, disable the resizable plugin
    if(jQueryelem.hasClass('ui-resizable') && jQueryelem.is('div')) {
      jQueryelem.resizable('destroy');
    } else if(jQueryelem.is('table')) {
      jQueryelem.tableResize('update');
    } else if(jQueryelem.is('th') || jQueryelem.is('td')) {
      jQueryelem.tableCellResize('disable');
    }
  }

}

/**
 * Create red border around the selected element.
 * @param object jQuerybase a jQuery element object
 */
divle_createRedBorder = function(jQuerybase, rmPrevs, extClass) {
  
  if(rmPrevs  == undefined || rmPrevs  == null) rmPrevs  = true;
  if(extClass == undefined || extClass == null) extClass = '';
  
  // remove the red box first
  if(rmPrevs) divle_removeRedBorder();

  if(jQuerybase.is("body"))
  {
	  jQuerybase.data("prevCssBorder", jQuerybase.css("border"));
	  jQuerybase.css("border", "2px solid #ff0000");
	  return;
  }
  
  jQuerybase.addClass('divle-redbord-owner');
  
  var jQueryredborder = jQuery('<div class="divlayer-red-border '+extClass+'" />').data('element', jQuerybase);
  divle_updateRedBorders(jQueryredborder);
}

/**
 * update red borders
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_updateRedBorders = function(redBordEs)
{
	if(redBordEs == undefined || redBordEs == null) redBordEs = '.divlayer-red-border';
	
	$(redBordEs).each(function(){
		var redBordE   = $(this);
		var selElement = redBordE.data('element');
		var classStr   = redBordE.attr('class');
		
		redBordE.remove();
		var offset = selElement.offset();
		
		// prepare border's placeholder
		var newRedBordE = jQuery('<div class="'+classStr+'" />')
		    .data('element', selElement)
		    .width(selElement.outerWidth())
		    .height(selElement.outerHeight())
		    .offset(offset)
		    .css('position', 'absolute')
		    .appendTo('body');
	});
}

/**
 * Remove any red border
 */
divle_removeRedBorder = function() {
  var bodyE = $('body');
  bodyE.css('border', ''); // @todo arif: populate body's border style if changed by editor
  var bodyPrevCssBorder = bodyE.data('prevCssBorder');
  if(!divle_isEmpty(bodyPrevCssBorder))
  {
	  bodyE.css('border', prevCssBorder);
	  bodyE.data('prevCssBorder', '');
  }
	  
  jQuery('.divlayer-red-border').each(function(){
	  $(this).data('element').removeClass('divle-redbord-owner');
	  $(this).remove();
  });
}

function divle_isEmpty (mixed_var)
{
    var key;
     if (mixed_var === "" || mixed_var === 0 || mixed_var === "0" || mixed_var === null || mixed_var === false || typeof mixed_var === 'undefined') {
        return true;
    }
 
    if (typeof mixed_var == 'object') {
    	for (key in mixed_var) {
            return false;
        }
        return true;
    } 
    return false;
}

/**
 * populate theme group versions
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_populateThemeGroupVersions = function()
{
    var template      = divle_getTemplateInThemeDropdn();
    var parStylesheet = divle_getParStylesheetInThemeDropdn();
    var grpVerSelO    = jQuery('#divlayer-input-theme-version');
    
    grpVerSelO.html('');
    grpVerSelO.append('<option value=""> </option>');
    
    for(var i = 0; i < divle_themeGroups.length; i++)
    {
    	var themeGroup = divle_themeGroups[i];
    	
    	if(themeGroup['template'] == template)
    	{
    		if(parStylesheet.length == 0)
    		{
        		for(var j = 0; j < themeGroup['themes'].length; j++)
        		{
        			var theme = themeGroup['themes'][j];
        			grpVerSelO.append('<option value="'+theme['stylesheet']+'">'+theme['group_version']+'</option>');
        		}
    		}
    		else
    		{
        		for(var j = 0; j < themeGroup['child_groups'].length; j++)
        		{
        			var childGroup = themeGroup['child_groups'][j];
        			
        			if(childGroup['par_stylesheet'] == parStylesheet)
        			{
        				for(var k = 0; k < childGroup['themes'].length; k++)
        				{
        					var theme = childGroup['themes'][k];
        					
        					grpVerSelO.append('<option value="'+theme['stylesheet']+'">'+theme['group_version']+'</option>');
        				}
        			}
        		}
    		}
    	}
    }
}

/**
 * switch button view to as loading
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_switchBtnViewToLoading = function(obj) {
	jQuery(obj).find('.divlayer-nav-images').css('background-image', 'url("'+divle_pluginUrl+'/images/ajax-loader-2.gif")');
}

/**
 * get parameter (GET) from URL
 */
divle_getURLParameter = function(name) {
    return decodeURI((RegExp(name + '=' + '(.+?)(&|$)').exec(location.search)||[,null])[1]);
}

divle_runEditorScripts = function() {
    // set the element as buttonset ui
    jQuery('.divlayer-buttonset').buttonset();

    // set the element as button ui
    jQuery('.divlayer-buttonset button').button();

    // set the element as tabs ui
    jQuery('#divlayer-property-tabs').tabs();

    // the up arrow button is clicked
    // get parent of currently selected element
    jQuery('#divlayer-arrow-up').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      // the button is disabled, no further action
      if(jQuery(this).hasClass('disabled')) return false;

      // get selected element
      var $dialog = jQuery('#divlayer-editor').parent();
      var $element = $dialog.data('element');

      // get selected element's parent node
      var $parent = $element.parent();

      // the selected element has no parent, no further action
      if($parent.length == 0) return false;

      // select parent of selected element
      divle_highlightElement($parent);

      return false;
    });

    // the down arrow button is clicked
    // get first children of currently selected element
    jQuery('#divlayer-arrow-down').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      // the button is disabled, no further action
      if(jQuery(this).hasClass('disabled')) return false;

      // get selected element
      var $dialog = jQuery('#divlayer-editor').parent();
      var $element = $dialog.data('element');

      // get selected element's first child
      var $child = $element.children(':first');

      // no element found, no further action
      if($child.length == 0) return false;

      // select first child of selected element
      divle_highlightElement($child);

      return false;
    });

    // the left arrow button is clicked
    // get previous sibling of currently selected element
    jQuery('#divlayer-arrow-left').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      // the button is disabled, no further action
      if(jQuery(this).hasClass('disabled')) return false;

      // get check button element
      var $check = jQuery('#divlayer-action-check');

      // get currently selected element
      var $dialog = jQuery('#divlayer-editor').parent();
      var $element = $dialog.data('element');

      if($check.hasClass('active')) {
        // the check button is active
        // select previous sibling with same class name

        if($dialog.data('element-selectors') != undefined && $dialog.data('element-selectors').length > 0) {
            // there are element selectors defined at divlayer dialog's data, use the list
            var elementSelectors = $dialog.data('element-selectors');
        } else {
            // use list of element selectors by find currently selected element's related element
            var elementSelectors = divle_getRelatedByClassSelectors($element, true);
        }

        // get css element selector
        var selector = divle_getElementSelector($element);

        // check the position of element selector in the list
        var selectorIndex = $.inArray(selector, elementSelectors);

        if(selectorIndex != -1) {
          // the selector is in the list

          // get another selector with lower index
          var prevSelectorIndex = selectorIndex - 1;
          if(prevSelectorIndex == -1) {
            prevSelectorIndex = elementSelectors.length - 1;
          }
          var prevSelector = elementSelectors[prevSelectorIndex];

          // select the element
          var $prev = $(prevSelector);
          divle_switchBtnViewToLoading('#divlayer-arrow-left');
          divle_highlightElement($prev);

          return false;
        }

      } else {
        // the check button is inactive
        // select previous sibling
        var $prev = $element.prev();
        if($prev.length == 0 && $element.is(':last-child') == false) {
          $prev = $element.siblings(':last-child');
        }

        // no element found, no further action
        if($prev.length == 0) return false;

        // select the element
        divle_highlightElement($prev);

        return false;
      }
    });

    // the right arrow button is clicked
    // get next sibling of currently selected element
    jQuery('#divlayer-arrow-right').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      // the button is disabled, no further action
      if(jQuery(this).hasClass('disabled')) return false;

      // get check button element
      var $check = jQuery('#divlayer-action-check');

      // get currently selected element
      var $dialog = jQuery('#divlayer-editor').parent();
      var $element = $dialog.data('element');

      if($check.hasClass('active')) {
        // the check button is active
        // select next sibling with same class name

        if($dialog.data('element-selectors') != undefined && $dialog.data('element-selectors').length > 0) {
            // there are element selectors defined at divlayer dialog's data, use the list
            var elementSelectors = $dialog.data('element-selectors');
        } else {
            // use list of element selectors by find currently selected element's related element
            var elementSelectors = divle_getRelatedByClassSelectors($element, true);
        }

        // get css element selector
        var selector = divle_getElementSelector($element);

        // check the position of element selector in the list
        var selectorIndex = $.inArray(selector, elementSelectors);

        if(selectorIndex != -1) {
          // the selector is in the list

          // get another selector with higher index
          var nextSelectorIndex = selectorIndex + 1;
          if(elementSelectors[nextSelectorIndex] == undefined) {
            nextSelectorIndex = 0;
          }
          var nextSelector = elementSelectors[nextSelectorIndex];

          // select the element
          var $next = $(nextSelector);
          divle_switchBtnViewToLoading('#divlayer-arrow-right');
          divle_highlightElement($next);

          return false;
        }
      } else {
        // the check button is inactive
        // select next sibling
        var $next = $element.next();
        if($next.length == 0 && $element.is(':first-child') == false) {
          $e = $element;
          $next = $element.siblings(':first-child');
        }

        // no element found, no further action
        if($next.length == 0) return false;

        // select the element
        divle_highlightElement($next);
        return false;
      }
    });

    // the check button is clicked
    // switch the element selection behavior
    jQuery('#divlayer-action-check').click(function(e){
      // prevent event's default behavior
      e.preventDefault();

      // the button is disabled, no further action
      if(jQuery(this).hasClass('disabled')) return false;

      var $this = jQuery(this);

      // get currently selected element
      var $dialog = jQuery('#divlayer-editor').parent();
      var $element = $dialog.data('element');

      if($this.hasClass('active') == false) {
        // set the button to active, get element selectors by class name

        // get list of of element selectors by find currently selected element's related element
        var elementSelectors = divle_getRelatedByClassSelectors($element, true);

        if(elementSelectors.length == 0) {
          // no related element selectors found

          $dialog.data('element-selectors', undefined);
          $this.removeClass('active');

          if(!($element.is('body') || $element.is('html'))) {
              // if selected element is not "body" or "html", enable up arrow button
              jQuery('#divlayer-arrow-up').removeClass('disabled');
          }
          if($element.children().length != 0) {
              // if selected element has children, enable down arrow button
              jQuery('#divlayer-arrow-down').removeClass('disabled');
          }

          alert('Element is not member of any classes.');
          return false;
        }

        // save the list to divlayer dialog's data
        $dialog.data('element-selectors', elementSelectors);

        // set the button to active
        $this.addClass('active');

        // disabled up and down arrow button
        jQuery('#divlayer-arrow-up').addClass('disabled');
        jQuery('#divlayer-arrow-down').addClass('disabled');

        // get element's classes
        var classes = divle_getElementClasses($element);

        // convert the list into string
        var classSelector = divle_stringifyClassNames(classes);

        // set the value of element selector input field with the class selector
        jQuery('#divlayer-search-element').val(classSelector);

        // set the class selector list to divlayer dialog's data
        $dialog.data('class-selector', classSelector);
        
        divle_openMultiSelect();
        
      } else {
        // set the button to inactive, get element selectors by default behavior

        // set divlayer dialog's data to default
        $dialog.data('element-selectors', []);
        $dialog.data('class-selector', '');

        // get css element selector of currently selected element
        var elementSelector = divle_getElementSelector($dialog.data('element'));

        // set the value of element selector input field with the css selector
        jQuery('#divlayer-search-element').val(elementSelector);

        // make the button inactive
        $this.removeClass('active');

        if(!($element.is('body') || $element.is('html')))  {
            // if selected element is not "body" or "html", enable up arrow button
            jQuery('#divlayer-arrow-up').removeClass('disabled');
        }
        if($element.children().length != 0) {
            // if selected element has children, enable down arrow button
            jQuery('#divlayer-arrow-down').removeClass('disabled');
        }
        
        divle_closeMultiSelect();
      }

      return false;
    });

    // emulate click on search button if "enter" key is pressed in the search input
    jQuery('#divlayer-search-element').keyup(function(e){
        if(e.keyCode == 13) {
          jQuery('#divlayer-action-search').trigger('click');
        }
    });

    // search the element by its css selector
    jQuery('#divlayer-action-search').click(function(e){
        // prevent event's default behavior
        e.preventDefault();

        // get the css selector
        var $search = jQuery('#divlayer-search-element');
        var selector = $.trim($search.val());

        // no css selector specified, no further action
        if(selector == '') return false;

        // get jQuery element by css selector
        var $selector = $(selector);

        if($selector.length == 0) {
          // no element found

          alert('No element match with given selector.');

          // if the field has previous css selector, revert it back
          var prevSelector = jQuery('#divlayer-dialog').data('element-selector');
          if(prevSelector != undefined || prevSelector != '') {
              $search.val(prevSelector);
          }
          return false;
        }

        if(divle_isValidClassSelector(selector)) {
            // if the selector is a valid css class select
            // set divlayer dialog's data of class selector
            jQuery('#divlayer-dialog')
                .data('class-selector', selector);
        }

        // only get single element
        $selector = $($selector.get(0));

        // select the element
        divle_highlightElement($selector);

        return false;
    });

    // set "change" event for css properties form fields
    jQuery('.divlayer-css-value-input').live('change', function(){
        // get current css property value
        var $this = jQuery(this);
        var property = $this.data('property');
        var value = $this.val();

        if($('#divlayer-action-check').hasClass('active'))
        {
        	var multiselSelectors = jQuery('#divlayer-dialog').data('multisel-selectors');
        	$.each(multiselSelectors, function(i, selector){
        		$(selector).css(property, value);
        	});
        }
        else
        {
            // get current element
        	var $element = jQuery('#divlayer-dialog').data('element');
            // apply new css property value to the element
            $element.css(property, value);
        }
        
        divle_updateRedBorders();
    });

    // parse css properties tabs and form fields
    divle_parseCssPropertiesTabs();
}

/**
 * open multi select as new
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_openMultiSelect = function()
{
    var divleDialog      = $('#divlayer-dialog');
	var elementSelectors = divleDialog.data('element-selectors');
    var currElemSelector = divleDialog.data('element-selector');
    var multiselSelectrs = divleDialog.data('multisel-selectors');
    
    if($.inArray(currElemSelector, multiselSelectrs) < 0) multiselSelectrs.push(currElemSelector);
    
    var multiSelContTable = $('#divlayer-multi-select-nav .divle-multi-sel-cont table');
    multiSelContTable.html('');
    var html = '';
    $.each(elementSelectors, function(i, selector) {
    	var checked  = ($.inArray(selector, multiselSelectrs) > -1) ? 'checked="checked"' : '';
    	html += '<tr class="divle-tr">'+
                  '<td class="divle-col-1"><input type="checkbox" name="multisel-selector" value="'+selector+'" '+checked+' /></td>'+
	              '<td class="divle-col-2">'+selector+'</td>'+
                '</tr>';
    });
    multiSelContTable.append(html);
    
    multiSelContTable.find('input:checkbox').change(function(){
    	divle_updateMultiSelect();
    });
    
    divle_updateMultiSelect();
    $('#divlayer-multi-select-nav').slideDown();
}

/**
 * update multi select
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_updateMultiSelect = function()
{
	var divleDialog       = $('#divlayer-dialog');
	var currElemSelector  = divleDialog.data('element-selector');
	var multiselSelectrs  = [];
	var multiSelContTable = $('#divlayer-multi-select-nav .divle-multi-sel-cont table');
	
	multiSelContTable.find('.divle-tr-checked').removeClass('divle-tr-checked').removeClass('divle-tr-check-curr');
	
	divle_removeRedBorder();
	
	multiSelContTable.find('.divle-tr input:checkbox:checked').each(function(){
		var selector = $(this).val();
		multiselSelectrs.push(selector);
		
		var redExtClass = 'divle-sec-red-border';
		var trExtClass  = '';
		if(currElemSelector == selector)
		{
			redExtClass = '';
			trExtClass  = ' divle-tr-check-curr';
		}
		$(this).parents('.divle-tr').addClass('divle-tr-checked'+trExtClass);
		
		divle_createRedBorder($(selector), false, redExtClass);
	});
	
    divleDialog.data('multisel-selectors', multiselSelectrs);
}

/**
 * close multi select
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_closeMultiSelect = function()
{
	var divleDialog      = $('#divlayer-dialog');
	var currElemSelector = divleDialog.data('element-selector');
	divle_createRedBorder($(currElemSelector));

	divleDialog.data('multisel-selectors', []);
    $('#divlayer-multi-select-nav').slideUp(function(){
    	$('#divlayer-multi-select-nav .divle-multi-sel-cont table').html('');
    });
}

/**
 * open help dialog
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_openHelp = function()
{
	var helpDialog = $('#divle-help-dialog');
	
	if(helpDialog.length < 1)
	{
		helpDialog = $('<div id="divle-help-dialog" ><div class="divle-iframe-cont"></div></div>').appendTo('body');
		
		helpDialog.dialog({
	      autoOpen: false,
	      title: 'Help on Div Layer Editor',
	      width: 800,
	      zIndex: 999999,
	      position: 'center',
	      close: function(){
		    $('#divle-help-dialog .divle-iframe-cont').html('');
	      }
	    });
	}
	
	var iframeHtml = '<iframe src="'+divle_helpIframeSrc+'" title="Help on Div Layer Editor" class="divle-help-iframe"></iframe>';	
	helpDialog.find('.divle-iframe-cont').html(iframeHtml);
	helpDialog.dialog('open');
}

/**
 * get selected template name in theme dropdown  
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_getTemplateInThemeDropdn = function()
{
	var valS = $('#divlayer-input-theme-name').val().split(':');
	
	return valS[0];
}

/**
 * get selected parent stylesheeet name in theme dropdown  
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
divle_getParStylesheetInThemeDropdn = function()
{
	var valS = $('#divlayer-input-theme-name').val().split(':');
	
	return (valS.length > 1) ? valS[1] : '';
}

jQuery(function(){
  // create dialog container
  jQuery('<div id="divlayer-dialog" />').appendTo('body');

  // iniitalized divlayer's dialog ui
  jQuery('#divlayer-dialog')
    .dialog({
      autoOpen: false,
      title: 'Div Layer Editor',
      width: 595,
      height: 500,
      zIndex: 999999,
      position: 'center',
      open: function(){
        // always load divlayer's dialog content when the dialog is opened
        var jQuerythis = jQuery(this);
        divle_reloadDialog();
      },
      close: function(){
        divle_closeDialog();
        var jQuerythis = jQuery(this);

        // clear out every stored data
        jQuerythis
          .empty()
          .data('element', [])
          .data('element-selectors', [])
          .data('element-selector', '')
          .data('class-selector', '')
          .data('multisel-selectors', []);
      }
    });
  
  // add help link into dle's title bar
  var helpHtml = '<a href="#" class="divlayer-button-help" title="Help"><span class="divlayer-nav-images divlayer-img-help"></span></a>';
  $('#divlayer-dialog').parents('.ui-dialog').find('.ui-dialog-titlebar .ui-dialog-titlebar-close').before(helpHtml);
  $('.divlayer-button-help').click(function(){
	  divle_openHelp();
	  return false;
  });
  
  var urlParamDivleEnable = divle_getURLParameter('divle-enable');
  
  if(divle_userHasAccess)
  {
	  var wpadmbarO = $('#wpadminbar');
	  if(wpadmbarO.length < 1) wpadmbarO = $('.divle-en-dis').parents("body > *");
	  wpadmbarO.addClass('divle-wpadminbar');
  }
  else if(divle_runAsDemo)
  {
	  if(urlParamDivleEnable == "true" || (typeof divle_isEnabled != 'undefined' && divle_isEnabled))
	  {
		  var abhaTitle = 'Disable Div Layer Editor';
		  var abhaText  = 'Disable DLE';
	  }
	  else
	  {
		  var abhaTitle = 'Enable Div Layer Editor';
		  var abhaText  = 'Enable DLE';
	  }
	  
	  var adminbarHtml = '<div class="divle-wpadminbar divle-wpadminbar-demo">'
		  					+'<div class="divle-en-dis">'
		  						+'<a title="'+abhaTitle+'" target="_blank" href="#">'+abhaText+'</a>'
		  					+'</div>'
		  				+'</div>';
	  $('body').append(adminbarHtml);
  }
  
  if(urlParamDivleEnable == "false")
  {
	  divle_disable();
  }
  else if(urlParamDivleEnable == "true" || (typeof divle_isEnabled != 'undefined' && divle_isEnabled))
  {
	  divle_enable();
	  
	  var elementPath = jQuery.cookie('divle_open_for_element');
	  jQuery(elementPath).click();
	  jQuery.cookie('divle_open_for_element', null);
  }
  
  jQuery('.divle-en-dis').click(function(){
		if(divle_isEnabled) divle_disable();
		else                divle_enable();
		return false;
  });
});

})(jQuery);