<?php
// here's where functions defined
require_once 'libraries.php';

/**
 * Main router for divlayer actions.
 *
 * This file contains router for specific divlayer actions.
 *
 */

if(!divle_currUserHasAccess())
{
    echo json_encode(array(
        'success' => false, 
        'meta'    => array(
            'error'     => 'no_access', 
            'error_msg' => "Sorry, you have no access to use this plugin")));
    return;
}

// use default action if no action specified
$action = isset($_GET['action']) ? $_GET['action'] : ''; 

// prepare response data
$response = array(); 

// check for appropriate action
switch($action) {
    case 'set':
        // set current version of the theme
        
        $themeGroups       = divle_getThemeGroups();
        $previewThemeGroup = divle_getPreviewThemeGroup($themeGroups);
        $previewTheme      = $previewThemeGroup ? divle_getPreviewTheme($previewThemeGroup) : null;
        
        $retData = array();
        
        if(!$previewTheme OR $previewTheme['is_current'])
        {
            $response['success'] = true; // already set as current
            $response['meta']    = $retData;
        }
        else
        {
            $response['success'] = divle_setThemeAsCurrent($previewTheme, $retData);
            $response['meta']    = $retData;
        }
        
        break;

    case 'enable-dle':
        // enable div layer editor
        
        divle_redirectToEnableDivle();
        
        break;
        
    case 'disable-dle':
        // disable div layer editor
        
        divle_redirectToDisableDivle();
        
        break;

    case 'save':   // save changes of the theme
        
        $themeGroups        = divle_getThemeGroups();
        $selectedThemeGroup = divle_getPreviewThemeGroup($themeGroups);
        $selectedTheme      = $selectedThemeGroup ? divle_getPreviewTheme($selectedThemeGroup) : null;
        
        if(!$selectedThemeGroup OR !$selectedTheme)
        {
            $selectedThemeGroup = divle_getCurrThemeGroup($themeGroups);
            $selectedTheme      = divle_getCurrTheme($selectedThemeGroup);
        }
        
        $trigger = (
                       isset($_POST['trigger']) 
                   AND $_POST['trigger'] == 'save' 
                   AND (float)$selectedTheme['group_version'] > 1
                   ) 
                   ? 'save' 
                   : 'new';
        
        // get array of changes, use default (empty array) if no changes made
        $css = isset($_POST['css']) ? $_POST['css'] : array();
        $retData = array();
        
        if($trigger == 'new')
        {
            $response['success'] = divle_createNewThemeInThemeGroupFromTheme($css, $selectedThemeGroup, $selectedTheme, $retData);
            $response['meta']    = $retData;
        }
        else // save
        {
            $response['success'] = divle_updateTheme($selectedTheme, $css, $retData);
            $response['meta']    = $retData;
        }

        break;
        
    case 'delete':   // delete a theme
        
        $themeGroups       = divle_getThemeGroups();
        //$currThemeGroup    = divle_getCurrThemeGroup($themeGroups);
        //$currTheme         = divle_getCurrTheme($currThemeGroup);
        $previewThemeGroup = divle_getPreviewThemeGroup($themeGroups);
        $previewTheme      = $previewThemeGroup ? divle_getPreviewTheme($previewThemeGroup) : null;
        
        $retData = array();
        
        if(!$previewTheme OR $previewTheme['is_current'])
        {
            $response['success'] = false;
            $response['meta']    = array (
                'error'     => 'curr_version_cannot_be_del',
                'error_msg' => "You cannot delete current theme version. Please set other theme version as current and then try to delete this one."
            );
        }
        else
        {
            $response['success'] = divle_deleteTheme($previewTheme, $retData);
            $response['meta']    = $retData;
        }
        
        break;

    default:
        // default action, currently return nothing but error
        $response['success'] = false;
        $response['meta']    = array (
                'error'     => 'invalid_action',
                'error_msg' => "Invalid action requested"
            );
}

// print response in json format
echo json_encode($response);
