<?php   
$wpConfigPath = divle_getWpConfigPath();
// Sets up the WordPress Environment.
require_once "{$wpConfigPath}/wp-load.php";

// get current theme function
function dle_get_current_theme() {
	if ( $theme = get_option('current_theme') )
		return $theme;

	$themes = dle_get_themes();
	$current_theme = 'Twenty Eleven';

	if ( $themes ) {
		$theme_names = array_keys( $themes );
		$current_template = get_option( 'template' );
		$current_stylesheet = get_option( 'stylesheet' );

		foreach ( (array) $theme_names as $theme_name ) {
			if ( $themes[$theme_name]['Stylesheet'] == $current_stylesheet &&
					$themes[$theme_name]['Template'] == $current_template ) {
				$current_theme = $themes[$theme_name]['Name'];
				break;
			}
		}
	}

	update_option('current_theme', $current_theme);

	return $current_theme;
}

// get theme data function
function dle_get_theme_data( $theme_file ) {
	$default_headers = array(
		'Name' => 'Theme Name',
		'URI' => 'Theme URI',
		'Description' => 'Description',
		'Author' => 'Author',
		'AuthorURI' => 'Author URI',
		'Version' => 'Version',
		'Template' => 'Template',
		'Status' => 'Status',
		'Tags' => 'Tags'
		);

	$themes_allowed_tags = array(
		'a' => array(
			'href' => array(),'title' => array()
			),
		'abbr' => array(
			'title' => array()
			),
		'acronym' => array(
			'title' => array()
			),
		'code' => array(),
		'em' => array(),
		'strong' => array()
	);

	$theme_data = get_file_data( $theme_file, $default_headers, 'theme' );

	$theme_data['Name'] = $theme_data['Title'] = wp_kses( $theme_data['Name'], $themes_allowed_tags );

	$theme_data['URI'] = esc_url( $theme_data['URI'] );

	$theme_data['Description'] = wptexturize( wp_kses( $theme_data['Description'], $themes_allowed_tags ) );

	$theme_data['AuthorURI'] = esc_url( $theme_data['AuthorURI'] );

	$theme_data['Template'] = wp_kses( $theme_data['Template'], $themes_allowed_tags );

	$theme_data['Version'] = wp_kses( $theme_data['Version'], $themes_allowed_tags );

	if ( $theme_data['Status'] == '' )
		$theme_data['Status'] = 'publish';
	else
		$theme_data['Status'] = wp_kses( $theme_data['Status'], $themes_allowed_tags );

	if ( $theme_data['Tags'] == '' )
		$theme_data['Tags'] = array();
	else
		$theme_data['Tags'] = array_map( 'trim', explode( ',', wp_kses( $theme_data['Tags'], array() ) ) );

	if ( $theme_data['Author'] == '' ) {
		$theme_data['Author'] = $theme_data['AuthorName'] = __('Anonymous');
	} else {
		$theme_data['AuthorName'] = wp_kses( $theme_data['Author'], $themes_allowed_tags );
		if ( empty( $theme_data['AuthorURI'] ) ) {
			$theme_data['Author'] = $theme_data['AuthorName'];
		} else {
			$theme_data['Author'] = sprintf( '<a href="%1$s" title="%2$s">%3$s</a>', $theme_data['AuthorURI'], esc_attr__( 'Visit author homepage' ), $theme_data['AuthorName'] );
		}
	}

	return $theme_data;
}

// get themes function
function dle_get_themes() {
	global $wp_themes, $wp_broken_themes;

	if ( isset($wp_themes) )
		return $wp_themes;

	if ( !$theme_files = search_theme_directories() )
		return false;

	asort( $theme_files );

	$wp_themes = array();

	foreach ( (array) $theme_files as $theme_file ) {
		$theme_root = $theme_file['theme_root'];
		$theme_file = $theme_file['theme_file'];

		if ( !is_readable("$theme_root/$theme_file") ) {
			$wp_broken_themes[$theme_file] = array('Name' => $theme_file, 'Title' => $theme_file, 'Description' => __('File not readable.'));
			continue;
		}

		$theme_data = dle_get_theme_data("$theme_root/$theme_file");

		$name        = $theme_data['Name'];
		$title       = $theme_data['Title'];
		$description = wptexturize($theme_data['Description']);
		$version     = $theme_data['Version'];
		$author      = $theme_data['Author'];
		$template    = $theme_data['Template'];
		$stylesheet  = dirname($theme_file);

		$screenshot = false;
		foreach ( array('png', 'gif', 'jpg', 'jpeg') as $ext ) {
			if (file_exists("$theme_root/$stylesheet/screenshot.$ext")) {
				$screenshot = "screenshot.$ext";
				break;
			}
		}

		if ( empty($name) ) {
			$name = dirname($theme_file);
			$title = $name;
		}

		$parent_template = $template;

		if ( empty($template) ) {
			if ( file_exists("$theme_root/$stylesheet/index.php") )
				$template = $stylesheet;
			else
				continue;
		}

		$template = trim( $template );

		if ( !file_exists("$theme_root/$template/index.php") ) {
			$parent_dir = dirname(dirname($theme_file));
			if ( file_exists("$theme_root/$parent_dir/$template/index.php") ) {
				$template = "$parent_dir/$template";
				$template_directory = "$theme_root/$template";
			} else {
				/**
				 * The parent theme doesn't exist in the current theme's folder or sub folder
				 * so lets use the theme root for the parent template.
				 */
				if ( isset($theme_files[$template]) && file_exists( $theme_files[$template]['theme_root'] . "/$template/index.php" ) ) {
					$template_directory = $theme_files[$template]['theme_root'] . "/$template";
				} else {
					if ( empty( $parent_template) )
						$wp_broken_themes[$name] = array('Name' => $name, 'Title' => $title, 'Description' => __('Template is missing.'), 'error' => 'no_template');
					else
						$wp_broken_themes[$name] = array('Name' => $name, 'Title' => $title, 'Description' => sprintf( __('The parent theme is missing. Please install the "%s" parent theme.'),  $parent_template ), 'error' => 'no_parent', 'parent' => $parent_template );
					continue;
				}

			}
		} else {
			$template_directory = trim( $theme_root . '/' . $template );
		}

		$stylesheet_files = array();
		$template_files = array();

		$stylesheet_dir = @ dir("$theme_root/$stylesheet");
		if ( $stylesheet_dir ) {
			while ( ($file = $stylesheet_dir->read()) !== false ) {
				if ( !preg_match('|^\.+$|', $file) ) {
					if ( preg_match('|\.css$|', $file) )
						$stylesheet_files[] = "$theme_root/$stylesheet/$file";
					elseif ( preg_match('|\.php$|', $file) )
						$template_files[] = "$theme_root/$stylesheet/$file";
				}
			}
			@ $stylesheet_dir->close();
		}

		$template_dir = @ dir("$template_directory");
		if ( $template_dir ) {
			while ( ($file = $template_dir->read()) !== false ) {
				if ( preg_match('|^\.+$|', $file) )
					continue;
				if ( preg_match('|\.php$|', $file) ) {
					$template_files[] = "$template_directory/$file";
				} elseif ( is_dir("$template_directory/$file") ) {
					$template_subdir = @ dir("$template_directory/$file");
					if ( !$template_subdir )
						continue;
					while ( ($subfile = $template_subdir->read()) !== false ) {
						if ( preg_match('|^\.+$|', $subfile) )
							continue;
						if ( preg_match('|\.php$|', $subfile) )
							$template_files[] = "$template_directory/$file/$subfile";
					}
					@ $template_subdir->close();
				}
			}
			@ $template_dir->close();
		}

		//Make unique and remove duplicates when stylesheet and template are the same i.e. most themes
		$template_files = array_unique($template_files);
		$stylesheet_files = array_unique($stylesheet_files);

		$template_dir = $template_directory;
		$stylesheet_dir = $theme_root . '/' . $stylesheet;

		if ( empty($template_dir) )
			$template_dir = '/';
		if ( empty($stylesheet_dir) )
			$stylesheet_dir = '/';

		// Check for theme name collision.  This occurs if a theme is copied to
		// a new theme directory and the theme header is not updated.  Whichever
		// theme is first keeps the name.  Subsequent themes get a suffix applied.
		// The Twenty Eleven, Twenty Ten, Default and Classic themes always trump
		// their pretenders.
		if ( isset($wp_themes[$name]) ) {
			$trump_cards = array(
				'classic'      => 'WordPress Classic',
				'default'      => 'WordPress Default',
				'twentyten'    => 'Twenty Ten',
				'twentyeleven' => 'Twenty Eleven',
			);
			if ( isset( $trump_cards[ $stylesheet ] ) && $name == $trump_cards[ $stylesheet ] ) {
				// If another theme has claimed to be one of our default themes, move
				// them aside.
				$suffix = $wp_themes[$name]['Stylesheet'];
				$new_name = "$name/$suffix";
				$wp_themes[$new_name] = $wp_themes[$name];
				$wp_themes[$new_name]['Name'] = $new_name;
			} else {
				$name = "$name/$stylesheet";
			}
		}

		$theme_roots[$stylesheet] = str_replace( WP_CONTENT_DIR, '', $theme_root );
		$wp_themes[$name] = array(
			'Name' => $name,
			'Title' => $title,
			'Description' => $description,
			'Author' => $author,
			'Author Name' => $theme_data['AuthorName'],
			'Author URI' => $theme_data['AuthorURI'],
			'Version' => $version,
			'Template' => $template,
			'Stylesheet' => $stylesheet,
			'Template Files' => $template_files,
			'Stylesheet Files' => $stylesheet_files,
			'Template Dir' => $template_dir,
			'Stylesheet Dir' => $stylesheet_dir,
			'Status' => $theme_data['Status'],
			'Screenshot' => $screenshot,
			'Tags' => $theme_data['Tags'],
			'Theme Root' => $theme_root,
			'Theme Root URI' => str_replace( WP_CONTENT_DIR, content_url(), $theme_root ),
		);
	}

	unset($theme_files);

	/* Store theme roots in the DB */
	if ( get_site_transient( 'theme_roots' ) != $theme_roots )
		set_site_transient( 'theme_roots', $theme_roots, 7200 ); // cache for two hours
	unset($theme_roots);

	/* Resolve theme dependencies. */
	$theme_names = array_keys( $wp_themes );
	foreach ( (array) $theme_names as $theme_name ) {
		$wp_themes[$theme_name]['Parent Theme'] = '';
		if ( $wp_themes[$theme_name]['Stylesheet'] != $wp_themes[$theme_name]['Template'] ) {
			foreach ( (array) $theme_names as $parent_theme_name ) {
				if ( ($wp_themes[$parent_theme_name]['Stylesheet'] == $wp_themes[$parent_theme_name]['Template']) && ($wp_themes[$parent_theme_name]['Template'] == $wp_themes[$theme_name]['Template']) ) {
					$wp_themes[$theme_name]['Parent Theme'] = $wp_themes[$parent_theme_name]['Name'];
					break;
				}
			}
		}
	}

	return $wp_themes;
 }


if(is_multisite())
{
	// include ms related functions.
	require_once "{$wpConfigPath}/wp-admin/includes/ms.php";
	require_once "{$wpConfigPath}/wp-admin/includes/theme.php";
}
// include global configuration for divlayer editor plugin
require_once plugin_dir_path(dirname(__FILE__)).'config.php';

/**
 * Collection of functions for divlayer editor.
 *
 * Contains all functions required by divlayer editor plugin to work.
 */

/*
 * get all theme groups
 * 
 * @param  $filter = allowed | all
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getThemeGroups($filter = 'allowed')
{
	$themeGroups = array();
    
    $isDivleEnabled = (isset($_COOKIE['divle_enabled']) AND $_COOKIE['divle_enabled'] == 'true');
    
    $currThemeName     = dle_get_current_theme();
    $previewTemplate   = ($isDivleEnabled AND !empty($_COOKIE['divle_preview_theme_template']))   ? $_COOKIE['divle_preview_theme_template']   : null;
    $previewStylesheet = ($isDivleEnabled AND !empty($_COOKIE['divle_preview_theme_stylesheet'])) ? $_COOKIE['divle_preview_theme_stylesheet'] : null;
    
    $themes = dle_get_themes();
    
    foreach($themes as $theme1)
    {
        if(empty($theme1['Parent Theme']))
        {
            $themeGroup = array();
            $themeGroup['name']         = $theme1['Name'];
            $themeGroup['template']     = $theme1['Template'];
            $themeGroup['themes']       = array();
            $themeGroup['child_groups'] = array(); // child themes which are not by DLE
            $themeGroup['is_current']   = false;
            $themeGroup['is_preview']   = false;
            
            foreach($themes as $theme2)
            {
                $theme2IsChildOfGroup  = ($theme2['Parent Theme'] == $themeGroup['name']);
                $theme2IsParentOfGroup = (empty($theme2['Parent Theme']) AND $theme2['Name'] == $themeGroup['name']);
                
                if($theme2IsChildOfGroup || $theme2IsParentOfGroup)
                {
		            /*
		             * In this block
		             * for the group theme2 may be
		             * 1. parent theme
		             * 2. child theme made by DLE from 1
		             * 3. child theme not made by DLE
		             * 4. child theme made by DLE from 3
		             */
                	
                	$theme2['is_current'] = false;
                    $theme2['is_preview'] = false;
                    
                    if($currThemeName == $theme2['Name'])
                    {
                        $themeGroup['is_current'] = true;
                        $theme2['is_current']     = true;
                    }
                    
                    if(    $previewTemplate   != null 
                       AND $previewStylesheet != null
                       AND $previewTemplate   == $themeGroup['template']
                       AND $previewStylesheet == $theme2['Stylesheet']
                       )
                    {
                        $themeGroup['is_preview'] = true;
                        $theme2['is_preview']     = true;
                    }
                    
                    $theme2IsMadeByDLE = ($theme2['Author'] == 'Div Layer Editor');
                    $hasTagChildGroupTheme = in_array('child-group-theme', $theme2['Tags']);
                    
                    if(   $theme2IsParentOfGroup 
                       || (   $theme2IsChildOfGroup 
                           && $theme2IsMadeByDLE
                           && !$hasTagChildGroupTheme
                           )
                       )
                    {
                        // cond: 1 or 2
                        
                    	$theme2['group_version'] = $theme2IsChildOfGroup ? $theme2['Version'] : "1";
                        
                        $themeGroup['themes'][]  = $theme2;
                    }
                    else
                    {
                        // cond: 3 or 4
                        
                    	if(!$theme2IsMadeByDLE && !$hasTagChildGroupTheme)
                    	{
	                        // cond: 3
	                        
                    		$childGroup = divle_prepChildGroup($theme2, $themes, $currThemeName, $previewTemplate, $previewStylesheet);
                    		
	                        $themeGroup['child_groups'][] = $childGroup;
	                        
                            if($childGroup['is_current']) $themeGroup['is_current'] = true;
                            if($childGroup['is_preview']) $themeGroup['is_preview'] = true;
                    	}
                    }
                }
            }
            
            $themeGroups[]  = $themeGroup;
        }
    }
    
    if($filter == 'allowed') $themeGroups = divle_filterAllowedThemeGroups($themeGroups);
    
    //echo '<pre>'; print_r($themeGroups); echo '</pre>';
    return $themeGroups;
}

/*
 * prepare child group from given theme and themes
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_prepChildGroup($theme1, $themes, $currThemeName, $previewTemplate, $previewStylesheet)
{
    $childGroup = array();
    $childGroup['name']               = $theme1['Name'];
	$childGroup['template']           = $theme1['Template'];
	$childGroup['par_stylesheet']     = $theme1['Stylesheet'];
	$childGroup['par_stylesheet_dir'] = $theme1['Stylesheet Dir'];
	$childGroup['themes']             = array();
	$childGroup['is_current']         = false;
	$childGroup['is_preview']         = false;
	
	foreach($themes as $theme2)
	{
		$childTag = "par-stylesheet:{$childGroup['par_stylesheet']}";
		$theme2IsChildOfGroup  = (in_array($childTag, $theme2['Tags']));
		$theme2IsParentOfGroup = ($theme2['Stylesheet'] == $childGroup['par_stylesheet']);
		
		if($theme2IsChildOfGroup || $theme2IsParentOfGroup)
		{
            $theme2['is_current'] = false;
            $theme2['is_preview'] = false;
            
            if($currThemeName == $theme2['Name'])
            {
                $childGroup['is_current'] = true;
                $theme2['is_current']     = true;
            }
            
            if(    $previewTemplate   != null 
               AND $previewStylesheet != null
               AND $previewTemplate   == $childGroup['template']
               AND $previewStylesheet == $theme2['Stylesheet']
               )
            {
                $childGroup['is_preview'] = true;
                $theme2['is_preview']     = true;
            }
                    
            $theme2['group_version']      = $theme2IsChildOfGroup ? $theme2['Version'] : "1";
            $theme2['par_stylesheet']     = $childGroup['par_stylesheet'];
            $theme2['par_stylesheet_dir'] = $childGroup['par_stylesheet_dir'];
            $theme2['child_group_name']   = $childGroup['name'];
                    
            $childGroup['themes'][]  = $theme2;
		}
	}
	
	return $childGroup;
}

/*
 * filter theme groups by allowed themes
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_filterAllowedThemeGroups($themeGroups)
{
	$allowedThemes = is_multisite() ? get_allowed_themes() : dle_get_themes();
	
	$filteredThGrps = array();
	
	foreach($themeGroups as $themeGroup)
	{
		$filteredThs = array();
		foreach($themeGroup['themes'] as $theme)
		{
			if(divle_existThemeInThemes($theme, $allowedThemes))
			{
				$filteredThs[] = $theme;
			}
		}
		
		$filteredChildGroups = divle_filterAllowedChildGroups($themeGroup['child_groups'], $allowedThemes);
		
		if(count($filteredThs) > 0 || count($filteredChildGroups) > 0)
		{
			$themeGroup['themes']       = $filteredThs;
			$themeGroup['child_groups'] = $filteredChildGroups;
			$filteredThGrps[]           = $themeGroup;
		}
	}
	
	return $filteredThGrps;
}

/*
 * filter child groups by allowed themes
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_filterAllowedChildGroups($groups, $allowedThemes)
{
    $filteredGrps = array();
    
    foreach($groups as $group)
    {
        $filteredThs = array();
        foreach($group['themes'] as $theme)
        {
            if(divle_existThemeInThemes($theme, $allowedThemes))
            {
                $filteredThs[] = $theme;
            }
        }
        
        if(count($filteredThs) > 0)
        {
            $group['themes'] = $filteredThs;
            $filteredGrps[]  = $group;
        }
    }
    
    return $filteredGrps;
}

/*
 * check whether a theme exists in a list of themes
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_existThemeInThemes($searchTheme, $themes)
{
	$exist = false;
	
	foreach($themes as $theme)
	{
		if($searchTheme['Template'] == $theme['Template'] AND $searchTheme['Stylesheet'] == $theme['Stylesheet'])
		{
			$exist = true;
			break;
		}
	}
	
	return $exist;
}

/*
 * get current theme group
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getCurrThemeGroup($themeGroups = null)
{
    if($themeGroups == null) $themeGroups = divle_getThemeGroups();
    
    $currThemeGroup = null;
    
    foreach($themeGroups as $themeGroup)
    {
        if($themeGroup['is_current'])
        {
            $currThemeGroup = $themeGroup;
            break;
        }
    }
    
    return $currThemeGroup;
}

/*
 * get current child group
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getCurrChildGroup($currThemeGroup = null)
{
	if($currThemeGroup == null) $currThemeGroup = divle_getCurrThemeGroup();
	
    $currChildGroup = null;
    
    foreach($currThemeGroup['child_groups'] as $childGroup)
    {
        if($childGroup['is_current'])
        {
            $currChildGroup = $childGroup;
            break;
        }
    }
    
    return $currChildGroup;
}

/*
 * get current theme
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getCurrTheme($currThemeGroup = null)
{
    if($currThemeGroup == null) $currThemeGroup = divle_getCurrThemeGroup();
    
    $currTheme = null;
    
    foreach($currThemeGroup['themes'] as $theme)
    {
        if($theme['is_current'])
        {
            $currTheme = $theme;
            break;
        }
    }
    
    if(!$currTheme)
    {
	    $currChildGroup = divle_getCurrChildGroup($currThemeGroup);
	    
	    if($currChildGroup)
	    {
		    foreach($currChildGroup['themes'] as $theme)
		    {
		        if($theme['is_current'])
		        {
		            $currTheme = $theme;
		            break;
		        }
		    }
	    }
    }
    
    return $currTheme;
}

/*
 * get preview theme group
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getPreviewThemeGroup($themeGroups = null)
{
    if($themeGroups == null) $themeGroups = divle_getThemeGroups();
    
    $previewThemeGroup = null;
    
    foreach($themeGroups as $themeGroup)
    {
        if($themeGroup['is_preview'])
        {
            $previewThemeGroup = $themeGroup;
            break;
        }
    }
    
    return $previewThemeGroup;
}

/*
 * get preview child group
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getPreviewChildGroup($previewThemeGroup = null)
{
    if($previewThemeGroup == null) $previewThemeGroup = divle_getPreviewThemeGroup();
    
    $previewChildGroup = null;
    
    foreach($previewThemeGroup['child_groups'] as $childGroup)
    {
        if($childGroup['is_preview'])
        {
            $previewChildGroup = $childGroup;
            break;
        }
    }
    
    return $previewChildGroup;
}

/*
 * get preview theme
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getPreviewTheme($previewThemeGroup = null)
{
    if($previewThemeGroup == null) $previewThemeGroup = divle_getPreviewThemeGroup();
    
    $previewTheme = null;
    
    if($previewThemeGroup)
    {
        foreach($previewThemeGroup['themes'] as $theme)
        {
            if($theme['is_preview'])
            {
                $previewTheme = $theme;
                break;
            }
        }
        
	    if(!$previewTheme)
	    {
	        $previewChildGroup = divle_getPreviewChildGroup($previewThemeGroup);
	        
	        if($previewChildGroup)
	        {
	            foreach($previewChildGroup['themes'] as $theme)
	            {
	                if($theme['is_preview'])
	                {
	                    $previewTheme = $theme;
	                    break;
	                }
	            }
	        }
	    }
    }
    
    return $previewTheme;
}

/*
 * get selected theme
 * find first for preview, if not found then find current
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getSelectedTheme($themeGroups = null)
{
	if($themeGroups == null) $themeGroups = divle_getThemeGroups();
	$selectedThemeGroup = divle_getPreviewThemeGroup($themeGroups);
	$selectedTheme      = $selectedThemeGroup ? divle_getPreviewTheme($selectedThemeGroup) : null;
        
	if(!$selectedThemeGroup OR !$selectedTheme)
	{
		$selectedThemeGroup = divle_getCurrThemeGroup($themeGroups);
		$selectedTheme      = divle_getCurrTheme($selectedThemeGroup);
	}
	
	return $selectedTheme;
}

/*
 * create new theme (child)in theme group (parent theme) from another theme 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_createNewThemeInThemeGroupFromTheme($css, $themeGroup, $fromTheme, &$retData)
{
	$parStylesheet = !empty($fromTheme['par_stylesheet']) ? $fromTheme['par_stylesheet'] : '';
	$createInChildGroup = !empty($parStylesheet);
	
    $newVersion = divle_calcNewThemeGroupVersion($themeGroup, $parStylesheet);
    if($newVersion === null)
    {
        $retData['error']     = 'new_version_calc_error';
        $retData['error_msg'] = "Could not calculate new version number";
        return false;
    }
    
    if($createInChildGroup)
    {
        $newThemeStylesheet = $fromTheme['par_stylesheet']     . DIVLE_CHILD_TH_DIR_NAME_SEP . $newVersion;
        $newThemeDir        = $fromTheme['par_stylesheet_dir'] . DIVLE_CHILD_TH_DIR_NAME_SEP . $newVersion;
    }
    else // create in theme group
    {
	    $newThemeStylesheet = $themeGroup['template']    . DIVLE_CHILD_TH_DIR_NAME_SEP . $newVersion;
	    $newThemeDir        = $fromTheme['Template Dir'] . DIVLE_CHILD_TH_DIR_NAME_SEP . $newVersion;
    }
    $newThemeJsDir = $newThemeDir . '/js';
    
    if(!mkdir($newThemeDir))
    {
        $retData['error']     = 'could_not_make_dir';
        $retData['error_msg'] = "Directory for new theme version could not be created. Please check themes directory's write permission.";
        return false;
    }
    
    $tmplThemeDir   = dirname(__FILE__) . '/../tmpl/child-theme';
    $tmplThemeJsDir = $tmplThemeDir . '/js';
    
    if($createInChildGroup)
    {
    	if(!divle_recursiveCopyContent($fromTheme['par_stylesheet_dir'], $newThemeDir))
    	{
	        $retData['error']     = 'could_not_copy_child_theme';
	        $retData['error_msg'] = "Could not copy child theme. Please check themes directory's write permission.";
	        return false;
    	}
    	
    	if(file_exists("{$newThemeDir}/functions.php")) rename("{$newThemeDir}/functions.php", "{$newThemeDir}/divlayereditorpar-functions.php");
    	if(file_exists("{$newThemeDir}/style.css"))     rename("{$newThemeDir}/style.css",     "{$newThemeDir}/divlayereditorpar-style.css");
    }
    
    if(!is_dir($newThemeJsDir)) mkdir($newThemeJsDir);    
    
    copy($tmplThemeDir . '/functions.php',   $newThemeDir . '/functions.php');
    //copy($tmplThemeJsDir . '/jquery.min.js', $newThemeJsDir . '/jquery.min.js');
    
    if($fromTheme['group_version'] > 1) // from child
        copy($fromTheme['Stylesheet Dir'] . '/js/div-layer-editor-style.js', $newThemeJsDir . '/div-layer-editor-style.js');
    else 
        copy($tmplThemeJsDir . '/div-layer-editor-style.js', $newThemeJsDir . '/div-layer-editor-style.js');
    
    // copy style.css
    $tags         = $createInChildGroup ? "par-stylesheet:{$fromTheme['par_stylesheet']}, child-group-theme" : "";
    $parThemeName = $createInChildGroup ? $fromTheme['child_group_name'] : $themeGroup['name'];
    $cssUrlImport = $createInChildGroup ? "divlayereditorpar-style.css"  : "../{$fromTheme['Template']}/style.css";
    
    $tmplStyleCssCont = file_get_contents($tmplThemeDir . '/style.css');
    $newStyleCssCont  = str_replace(array("{PARENT_THEME_NAME}", "{THEME_GROUP_VERSION}", "{PARENT_THEME_DIRECTORY}", "{TAGS}", "{CSS_URL_IMPORT}"), array($parThemeName, "{$newVersion}", "{$fromTheme['Template']}", $tags, $cssUrlImport), $tmplStyleCssCont);
    file_put_contents($newThemeDir . '/style.css', $newStyleCssCont);
    // end: copy style.css
    
    if(!divle_updateStyleJsFile($newThemeJsDir . '/div-layer-editor-style.js', $css, $retData))
    {
        return false;
    }
    
    // enable new created theme for the site which created it
    if(is_multisite())
    {
	    $allowedThemes = get_option('allowedthemes');
	    
	    if(!$allowedThemes)
	        $allowedThemes = array($newThemeStylesheet => true);
	    else
	        $allowedThemes[$newThemeStylesheet] = true;
	        
	    update_option('allowedthemes', $allowedThemes);
    }
    
    $retData['new_version']    = $newVersion;
    $retData['new_stylesheet'] = $newThemeStylesheet;
    return true;
}

/*
 * update a theme (child) 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_updateTheme($updTheme, $css, &$retData)
{
    if($updTheme['group_version'] < 2)
    {
        $retData['error']     = 'version_1_cannot_be_edited';
        $retData['error_msg'] = "Theme of version 1 (i.e. parent theme) cannot be edited";
        return false;
    }
    
    return divle_updateStyleJsFile($updTheme['Stylesheet Dir'] . '/js/div-layer-editor-style.js', $css, $retData);
}

/*
 * update a div-layer-editor-style.js file of child theme with additional css  
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_updateStyleJsFile($styleJsFile, $css, &$retData)
{
    $styleJsCont   = file_get_contents($styleJsFile);
    $styleJsContSs = explode("/**data**/", $styleJsCont);
    
    if(count($styleJsContSs) < 3)
    {
        $retData['error']     = 'invalid_style_js_file';
        $retData['error_msg'] = "The format of the style js file is not correct";
        return false;
    }
    
    $styleData = $styleJsContSs[1];

    $tmplStyleDataPartCont = file_get_contents(dirname(__FILE__) . '/../tmpl/child-theme/js/style-data-part.js');
    
    foreach($css as $elementPath => $elementCss)
    {
        $cssArrayStr = "";
        foreach($elementCss as $cssProp => $cssVal)
        {
            if(!empty($cssArrayStr)) $cssArrayStr .= "\n";
            $cssArrayStr .= "c['{$cssProp}'] = '{$cssVal}';";
        }
        
        $styleDataPart = str_replace(array("{ELEMENT_PATH}", "{CSS_ARRAY}"), array("{$elementPath}", "{$cssArrayStr}"), $tmplStyleDataPartCont);
        $styleData .= $styleDataPart;
    }
    
    $styleJsContSs[1] = $styleData;
    
    if(!file_put_contents($styleJsFile, implode("/**data**/", $styleJsContSs)))
    {
        $retData['error']     = 'could_not_write_to_style_js_file';
        $retData['error_msg'] = "Could not write to the style js file. Please check the file's write permission.";
        return false;
    }
    
    return true;
}

/*
 * set a theme as current 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_setThemeAsCurrent($actTheme, &$retData)
{
    switch_theme($actTheme['Template'], $actTheme['Stylesheet']);
    
    return true;
}

/*
 * delete a theme (child) 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_deleteTheme($delTheme, &$retData)
{
    if($delTheme['group_version'] < 2)
    {
        $retData['error']     = 'version_1_cannot_be_del';
        $retData['error_msg'] = "Theme of version 1 (i.e. parent theme) cannot be deleted";
        return false;
    }
    
    if(!divle_rrmdir($delTheme['Stylesheet Dir']))
    {
        $retData['error']     = 'could_not_del_dir';
        $retData['error_msg'] = "The theme version could not be deleted. Please check themes directory's write permission.";
        return false;
    }
    
    return true;
}

/**
 * remove a directory and it's contents
 * 
 * @return boolean
 */
function divle_rrmdir($dir) 
{
    $ret = true;
    
    if(is_dir($dir))
    {
        $objects = scandir($dir);
        foreach ($objects as $object)
        {
            if ($object != "." && $object != "..")
            {
                if(filetype($dir."/".$object) == "dir")
                {
                    if(!divle_rrmdir($dir."/".$object)) $ret = false;
                }
                else
                {
                    if(!unlink($dir."/".$object)) $ret = false;
                }
            }
        }
        reset($objects);
        if(!rmdir($dir)) $ret = false;
    }
    else
    {
        $ret = false;
    }
    
    return $ret;
}

/*
 * get path of wp-config.php file 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_getWpConfigPath($path = null)
{
    if($path == null) $path = __FILE__;
    
    $path   = str_replace("\\", "/", $path);
    $pathSs = explode('/', $path);
    
    $confPath = null;
    while(count($pathSs) > 0)
    {
        array_pop($pathSs);
        $confFile = implode('/', $pathSs)."/wp-config.php";
        if(@file_exists($confFile))
        {
            $confPath = dirname($confFile);
            break;
        }
    }
    
    return $confPath;
}

/*
 * check whether current user has access to use this plugin 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_currUserHasAccess()
{
    return current_user_can('manage_options');
}

/*
 * redirect to enable div layer editor 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_redirectToEnableDivle()
{
    wp_redirect(get_bloginfo('url') . '?divle-enable=true');
}

/*
 * redirect to disable div layer editor 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_redirectToDisableDivle()
{
    wp_redirect(get_bloginfo('url') . '?divle-enable=false');
}

/*
 * calculate new group version of theme 
 * for creating new theme (child) under a theme group 
 * 
 * @author md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_calcNewThemeGroupVersion($themeGroup, $parStylesheet = '')
{
    $allThemeGroups = divle_getThemeGroups('all');
    
    $themes = array();
    
    foreach($allThemeGroups as $allThemeGroup)
    {
    	if($allThemeGroup['template'] == $themeGroup['template'])
    	{
    		if(empty($parStylesheet))
    		{
    			$themes = $allThemeGroup['themes'];
    		}
    		else
    		{
    			$allChildGroups = $allThemeGroup['child_groups'];
    			foreach($allChildGroups as $allChildGroup)
    			{
    				if($allChildGroup['par_stylesheet'] == $parStylesheet)
    				{
    					$themes = $allChildGroup['themes'];
    					break;
    				}
    			}
    		}
    		
    		break;

    	}
    }
    
	$newVersion = 0;
    foreach($themes as $theme)
    {
        if($theme['group_version'] > $newVersion) $newVersion = $theme['group_version'];
    }
    $newVersion++;
    
    return (count($themes) > 0) ? $newVersion : null;
}

/*
 * recursively copy files and directories of a directory to another directory
 * merges the directories 
 * 
 * @return  boolean
 * @author  md. arifuzzaman <arif69eee@gmail.com>
 */
function divle_recursiveCopyContent($source, $dest)
{ 
    if(!is_dir($dest) AND !mkdir($dest)) return false; 
    
    $sourceHandle = opendir($source); 
    
    while($res = readdir($sourceHandle))
    { 
        if($res == '.' || $res == '..') continue; 
        
        if(is_dir("{$source}/{$res}"))
        { 
            if(!divle_recursiveCopyContent("{$source}/{$res}", "{$dest}/{$res}")) return false; 
        } 
        else 
        { 
            if(!copy("{$source}/{$res}", "{$dest}/{$res}")) return false; 
        } 
    }
    
    return true;
} 